#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include "i2c.h"
#include "lcd.h"
#include "keyb.h"
#include "MCP79410.h"

const char * WeekDays[] = {"  ", "pn", "wt", "sr", "cz", "pt", "so", "nd"};
	
const uint8_t customCharBell[8] = {
	0b00000,
	0b00100,
	0b01110,
	0b01110,
	0b01110,
	0b11111,
	0b00100,
	0b00000
};

volatile uint8_t alarmActivated = 0;
	
void setDateTime();
uint8_t getWeekDay(DateTime * dateTime);
void setAlarmTime(uint8_t * alarmHour, uint8_t * alarmMinute);
void startBlink();
void stopBlink();

int main(void) {
	i2cInit();
	keybInit();
	lcdInit();
	lcdInitPrintf();
	lcdDefineChar(customCharBell, 1);
	uint8_t alarmMinute = 0;
	uint8_t alarmHour = 0;
	//podcignicie PORTB2
	PORTB |= _BV(PORTB2);
	//wczenie przerwania INT2
	GICR |= _BV(INT2);
	//wcz przerwania globalnie
	sei();
	//PB0 jako wyjcie
	DDRB |= _BV(DDB0);
	//szybko migania
	OCR1A = 4000;
	TIMSK = _BV(OCIE1A);
	//skasuj ewentualny alarm
	rtcWriteRegister(MCP79410_ALM0WKDAY, 0);
	while(1){
		//pobranie aktualnego czasu
		DateTime dateTime;
		rtcReadDateTime(&dateTime);
		//wywietlenie czasu
		lcdGotoXY(0, 0);
		printf("%02d:%02d:%02d", dateTime.hours, dateTime.minutes, dateTime.seconds);
		//wywietlenie daty
		lcdGotoXY(0, 1);
		printf("%02d-%02d-%4d", dateTime.dayOfMonth, dateTime.month, dateTime.year);
		printf(" %s", WeekDays[dateTime.dayOfWeek]);
		//wywietlenie ikonki alarmu jeli wczony
		uint8_t control;
		rtcReadRegister(MCP79410_CONTROL, &control);
		if (control & _BV(MCP79410_ALM0EN)) {
			printf(" \x01");
		} else {
			printf("  ");
		}
		//odczyt klawiatury
		uint8_t key = 0;
		if ((key = readKeyboard())) {
			while(readKeyboard());
			if (key == 1) setDateTime();
			if (key == 2) setAlarmTime(&alarmHour, &alarmMinute);
			if (key == 3) {
				stopBlink();
			}
			//wyczenie alarmu, MFP=1
			if (key == 4) rtcWriteRegister(MCP79410_CONTROL, _BV(MCP79410_OUT));
		}
		if (alarmActivated) {
			alarmActivated = 0;
			//skasowanie alarmu
			rtcWriteRegister(MCP79410_ALM0WKDAY, 0);
			//pobranie aktualnego czasu
			rtcReadDateTime(&dateTime);
			if (dateTime.hours == alarmHour && dateTime.minutes == alarmMinute) {
				startBlink();
			}
		}
	}
}

ISR(TIMER1_COMPA_vect) {
	if (PORTB & _BV(PORTB0)) {
		PORTB &= ~_BV(PORTB0);
	} else {
		PORTB |= _BV(PORTB0);
	}
}

ISR(INT2_vect) {
	alarmActivated = 1;
}

void startBlink() {
	//CTC, preskaler 1024
	TCCR1B = _BV(CS12) | _BV(CS10) | _BV(WGM12);
}

void stopBlink() {
	TCCR1B = 0;
	PORTB &= ~_BV(PORTB0);
}

void setDateTime() {
	//wcz kursor
	lcdWriteCommand(LCD_COMMAND_ON_OFF | LCD_PARAM_ON_OFF_CURSOR | LCD_PARAM_ON_OFF_DISPLAY);
	//wczytaj czas
	lcdGotoXY(0, 0);
	printf("  :  :  ");
	lcdGotoXY(0, 0);
	const uint8_t timeFieldsSizes[] = {2, 2, 2};
	uint16_t time[3];
	readSeparatedNumbers(timeFieldsSizes, 3, time);
	//wczytaj dat
	lcdGotoXY(0, 1);
	printf("  -  -    ");
	lcdGotoXY(0, 1);
	const uint8_t dateFieldsSizes[] = {2, 2, 4};
	uint16_t date[3];
	readSeparatedNumbers(dateFieldsSizes, 3, date);
	DateTime newDateTime;
	newDateTime.hours = time[0];
	newDateTime.minutes = time[1];
	newDateTime.seconds = time[2];
	newDateTime.dayOfMonth = date[0];
	newDateTime.month = date[1];
	newDateTime.year = date[2];
	newDateTime.dayOfWeek = getWeekDay(&newDateTime);
	//zapisz nowy czas i dat
	rtcWriteDateTime(&newDateTime);
	//wycz kursor
	lcdWriteCommand(LCD_COMMAND_ON_OFF | LCD_PARAM_ON_OFF_DISPLAY);
}

void setAlarmTime(uint8_t * alarmHour, uint8_t * alarmMinute) {
	//wcz kursor
	lcdWriteCommand(LCD_COMMAND_ON_OFF | LCD_PARAM_ON_OFF_CURSOR | LCD_PARAM_ON_OFF_DISPLAY);
	//wyczy wywietlacz
	lcdWriteCommand(LCD_COMMAND_CLEAR);
	//wczytaj czas
	lcdGotoXY(0, 0);
	printf("  :  ");
	lcdGotoXY(0, 0);
	const uint8_t timeFieldsSizes[] = {2, 2};
	uint16_t time[2];
	readSeparatedNumbers(timeFieldsSizes, 2, time);
	*alarmHour = time[0];
	*alarmMinute = time[1];
	//zapisz nowy czas i dat
	rtcWriteMinuteAlarm(0);
	//wcz alarm
	rtcWriteRegister(MCP79410_CONTROL, _BV(MCP79410_ALM0EN));
	//wycz kursor
	lcdWriteCommand(LCD_COMMAND_ON_OFF | LCD_PARAM_ON_OFF_DISPLAY);
}

uint8_t getWeekDay(DateTime * dateTime) {
	int d = dateTime->dayOfMonth;
	int m = dateTime->month;
	int y = dateTime->year;
	uint8_t weekday  = (d += m < 3 ? y-- : y - 2, 23*m/9 + d + 4 + y/4- y/100 + y/400)%7;
	if (!weekday) weekday = 7;
	return weekday;
}